using UnityEngine;using System.Collections;namespace PluginIdea.Fluid{public class SSF_DepthRenderer : MonoBehaviour{public ParticleSystem particleSystem;public int particlesCount = 0;public float pointScale = 1.0f;public float pointRadius = 0.02f;public float blurScale = 1f;public int blurRadius = 10;public float blurDepthFalloff = 100.0f;public float minDepth = 0.0f;public float thickness = 1.0f;public float softness = 1.0f;public bool iaAlignParticleSize;public float hStrengh = 1;public float vStrengh = 1;public float vOffset = 0.5f;public float hBaseOffset = 1f;public float vBaseOffset = 1f;public float hExStrengh = 0f;private Material depthMaterial;private Material blurDepthMaterial;private Material thicknessMaterial;private ComputeBuffer posBuffer;private ComputeBuffer colorBuffer;private ComputeBuffer quadVerticesBuffer;private ComputeBuffer velocityBuffer;private Vector4[] particlePositions;private Vector3[] particlVelocities;private Color[] particleColors;private ParticleSystem.Particle[] m_particles;private Camera windowCamera;void Start(){windowCamera = Utils.GetWindowCamera();particleSystem = GetComponent<ParticleSystem>();CreateMaterials();CreateEsseialArrs();CreateBuffers();UpdateParticlesData();FillBufferData();CreateQuadBuffers();PushMaterialBuffer(depthMaterial);PushMaterialBuffer(thicknessMaterial);}private void CreateQuadBuffers(){quadVerticesBuffer = new ComputeBuffer(6, 16);quadVerticesBuffer.SetData(new[]{new Vector4(-0.5f, 0.5f),            new Vector4(0.5f, 0.5f),            new Vector4(0.5f, -0.5f),            new Vector4(0.5f, -0.5f),            new Vector4(-0.5f, -0.5f),            new Vector4(-0.5f, 0.5f),});}private void CreateEsseialArrs(){m_particles = new ParticleSystem.Particle[particleSystem.main.maxParticles];particlePositions = new Vector4[particleSystem.main.maxParticles];particlVelocities = new Vector3[particleSystem.main.maxParticles];;particleColors = new Color[particleSystem.main.maxParticles];;}private void CreateBuffers(){posBuffer = new ComputeBuffer(particleSystem.main.maxParticles, 16);colorBuffer = new ComputeBuffer(particleSystem.main.maxParticles, 16);velocityBuffer = new ComputeBuffer(particleSystem.main.maxParticles, 12);}private void CreateMaterials(){depthMaterial = new Material(SSF.dic_name2Shaders["PI/Depth"]);thicknessMaterial = new Material(SSF.dic_name2Shaders["PI/Thick"]);blurDepthMaterial = new Material(SSF.dic_name2Shaders["PI/DepthBlur"]);depthMaterial.hideFlags = HideFlags.HideAndDontSave;thicknessMaterial.hideFlags = HideFlags.HideAndDontSave;blurDepthMaterial.hideFlags = HideFlags.HideAndDontSave;}private void PushMaterialBuffer(Material targetMat){targetMat.SetBuffer("buf_Positions", posBuffer);targetMat.SetBuffer("buf_Colors", colorBuffer);targetMat.SetBuffer("buf_Vertices", quadVerticesBuffer);targetMat.SetBuffer("buf_Velocities", velocityBuffer);}void OnRenderObject(){if (Camera.current != Utils.CenterCamera && Camera.current != windowCamera) return;UpdateParticlesData();FillBufferData();DrawDepth();BlurDepth();DrawThickness();}private void UpdateParticlesData(){int particlesCount = particleSystem.GetParticles(m_particles);for (int i = 0;i < particlesCount;i++) UpdateParticleData(i);}void UpdateParticleData(int index){var particle = m_particles[index];particlePositions[index] = new Vector4(particle.position.x, particle.position.y, particle.position.z, particle.GetCurrentSize(particleSystem));particlVelocities[index] = particle.velocity;particleColors[index] = particle.GetCurrentColor(particleSystem);}void FillBufferData(){posBuffer.SetData(particlePositions);colorBuffer.SetData(particleColors);velocityBuffer.SetData(particlVelocities);}void DrawDepth(){depthMaterial.SetVector("_ShapePara1", new Vector4(hStrengh, vStrengh, vOffset));depthMaterial.SetVector("_ShapePara2", new Vector4(hBaseOffset, vBaseOffset, hExStrengh));depthMaterial.SetFloat("_PointRadius", pointRadius);depthMaterial.SetFloat("_PointScale", pointScale);PushMaterialBuffer(depthMaterial);Graphics.SetRenderTarget(SSF.m_depthTexture);GL.Clear(true, true, Color.white);depthMaterial.SetPass(0);Graphics.DrawProcedural(MeshTopology.Triangles, 6, particleSystem.particleCount);}void BlurDepth(){blurDepthMaterial.SetTexture("_DepthTex", SSF.m_depthTexture);blurDepthMaterial.SetInt("radius", blurRadius);blurDepthMaterial.SetFloat("minDepth", minDepth);blurDepthMaterial.SetFloat("blurDepthFalloff", blurDepthFalloff);blurDepthMaterial.SetTexture("_DepthTex", SSF.m_depthTexture);blurDepthMaterial.SetFloat("scaleX", 1.0f / 1024 * blurScale);blurDepthMaterial.SetFloat("scaleY", 0.0f);Graphics.Blit(SSF.m_depthTexture, SSF.m_blurredDepthTempTexture, blurDepthMaterial);blurDepthMaterial.SetTexture("_DepthTex", SSF.m_blurredDepthTempTexture);blurDepthMaterial.SetFloat("scaleX", 0.0f);blurDepthMaterial.SetFloat("scaleY", 1.0f / 1024 * blurScale);Graphics.Blit(SSF.m_blurredDepthTempTexture, SSF.m_blurredDepthTexture, blurDepthMaterial);}void DrawThickness(){thicknessMaterial.SetVector("_ShapePara1", new Vector4(hStrengh, vStrengh, vOffset));thicknessMaterial.SetVector("_ShapePara2", new Vector4(hBaseOffset, vBaseOffset, hExStrengh));thicknessMaterial.SetFloat("_PointRadius", pointRadius);thicknessMaterial.SetFloat("_PointScale", pointScale);thicknessMaterial.SetFloat("_Thickness", thickness);thicknessMaterial.SetFloat("_Softness", softness);PushMaterialBuffer(thicknessMaterial);Graphics.SetRenderTarget(SSF.m_thicknessTexture);GL.Clear(true, true, Color.black);thicknessMaterial.SetPass(0);Graphics.DrawProcedural(MeshTopology.Triangles, 6, particleSystem.particleCount);}void OnDestroy(){ReleaseBuffers();}void ReleaseBuffers(){posBuffer?.Release();colorBuffer?.Release();quadVerticesBuffer?.Release();velocityBuffer?.Release();}}}